@@ -8,11 +8,11 @@ getUsefulPredictors <- function(x) {
 
 
 ## Extract summary statistics (linear regression: Y~A within subgroup) ##
-lm_stats = function(summ.fit, Subgrps, s, alpha, noA){
+lm_stats = function(summ.fit, Subgrps, s, alpha, noA) {
   # Sample size #
   n.s <- length(summ.fit$residuals)
   coeff <- summ.fit$coefficients
-  if (dim(coeff)==1){
+  if (dim(coeff)==1) {
     est <- coeff[1,1]
     SE <- coeff[1,2]
     LCL <- est - qt(1-alpha/2, df=n.s-1)*SE
@@ -21,7 +21,7 @@ lm_stats = function(summ.fit, Subgrps, s, alpha, noA){
     summ <- data.frame( Subgrps = s, N = n.s, 
                        estimand = "E(Y)", est, SE, LCL, UCL, pval)
   }
-  if (dim(coeff)[1]>1){
+  if (dim(coeff)[1]>1) {
     L.mat = rbind( c(1,0), c(1,1), c(0,1) )
     est = L.mat %*% coeff[,1]
     SE = sqrt(  diag( L.mat %*% vcov(summ.fit) %*% t(L.mat) ) )
@@ -45,17 +45,17 @@ norm_norm <- function(PRISM.fit, alpha_ovrl, alpha_s,
     param.e <- param.dat[param.dat$estimand==e,]
     prior.mu <- param.e$est[param.e$Subgrps==0]
     prior.var <- scale * ( param.e$SE[param.e$Subgrps==0] )^2
-    looper <- function(s){
+    looper <- function(s) {
       ns <- param.e$N[param.e$Subgrps==s]
       obs.mu <- param.e$est[param.e$Subgrps==s]
       obs.var <- ( param.e$SE[param.e$Subgrps==s] )^2
       ### Mean/Var ##
-      if (s==0){
+      if (s==0) {
         mu_B <- obs.mu
         var_B <- obs.var
         alpha <- alpha_ovrl
       }
-      if (s>0){
+      if (s>0) {
         var_B <- ( 1/prior.var + 1/(obs.var) )^(-1)
         mu_B <- var_B * ( prior.mu/prior.var + obs.mu / (obs.var)    ) 
         alpha <- alpha_s
@@ -73,24 +73,26 @@ norm_norm <- function(PRISM.fit, alpha_ovrl, alpha_s,
     return(param.B)
   }
   bayes_param = NULL
-  for (e in unique(param.dat$estimand)){
+  for (e in unique(param.dat$estimand)) {
     param.B = get_posterior_ests(param.dat=param.dat, e = e)
     bayes_param = suppressWarnings( bind_rows(bayes_param, param.B) )
   }
-  param.dat <- left_join(param.dat, bayes_param, by=c("Subgrps", "estimand"))
-  bayes.sim = function(mu, sd, n=100000){
+  bayes_param$`Prob(>0)` = with(bayes_param, 1-pnorm(0, mean=est.bayes, sd=SE.bayes))
+  param.dat <- suppressWarnings(
+    left_join(param.dat, bayes_param, by=c("Subgrps", "estimand")))
+  bayes.sim = function(mu, sd, n=100000) {
     return( rnorm(n=n, mean = mu, sd = sd)  )
   }
   return( list(param.dat=param.dat, bayes.sim=bayes.sim) )
 }
 
 ## Generate bootstrap resampling indices ##
-bootstrap_indices = function(n, R, strata=NULL){
-  if (is.null(strata)){
+bootstrap_indices = function(n, R, strata=NULL) {
+  if (is.null(strata)) {
     indices <- sample.int(n, n * R, replace = TRUE)
     dim(indices) <- c(R, n)
   }
-  if (!is.null(strata)){
+  if (!is.null(strata)) {
     indices <- matrix(NA, R, n)
     for (s in unique(strata)) {
       sub_i <- seq_len(n)[strata == s]
@@ -101,19 +103,19 @@ bootstrap_indices = function(n, R, strata=NULL){
   return(indices)
 }
 ## Generate permutation resampling indices ##
-permute_indices = function(n, R, strata=NULL){
-  if (is.null(strata)){
+permute_indices = function(n, R, strata=NULL) {
+  if (is.null(strata)) {
     indices <- matrix(NA, R, n)
-    for (r in 1:R){
+    for (r in 1:R) {
       indices[r,] <- sample(n, size=n, replace=FALSE)
     }
   }
-  if (!is.null(strata)){
+  if (!is.null(strata)) {
     indices <- matrix(NA, R, n)
     for (s in unique(strata)) {
       sub_i <- seq_len(n)[strata == s]
       ns = length(sub_i)
-      for (r in 1:R){
+      for (r in 1:R) {
         indices[r, sub_i] <- sample(sub_i, replace=FALSE)
       }
     }
@@ -121,13 +123,13 @@ permute_indices = function(n, R, strata=NULL){
   return(indices)
 }
 ## Generate CV sampling folds ###
-CV_folds = function(n, R, strata=NULL){
-  if (is.null(strata)){
+CV_folds = function(n, R, strata=NULL) {
+  if (is.null(strata)) {
     folds <- sample(rep(1:R,ceiling(n/R))[1:n])
   }
-  if (!is.null(strata)){
+  if (!is.null(strata)) {
     folds = NULL
-    for (s in unique(strata)){
+    for (s in unique(strata)) {
       sub_i <- seq_len(n)[strata == s]
       ns = length(sub_i)
       folds.s <- sample(rep(1:R,ceiling(ns/R))[1:ns])
@@ -138,10 +140,10 @@ CV_folds = function(n, R, strata=NULL){
 }
 
 ## Coverage counter(for bootstrap calibration) ##
-coverage_counter = function(param.dat, alpha.mat){
+coverage_counter = function(param.dat, alpha.mat) {
   
   counts = NULL
-  counter = function(i){
+  counter = function(i) {
     alpha_s = alpha.mat$alpha_s[i]
     alpha_ovrl = alpha.mat$alpha_ovrl[i]
     param.dat$alpha = with(param.dat, ifelse(Subgrps==0, alpha_ovrl, alpha_s))
@@ -155,12 +157,12 @@ coverage_counter = function(param.dat, alpha.mat){
   counts = data.frame( do.call(rbind,counts) )
   return(counts)
 }
-calibrate_alpha = function(resamp_calib, alpha_ovrl, alpha_s){
+calibrate_alpha = function(resamp_calib, alpha_ovrl, alpha_s) {
   
   # Average across overall/subgroups and within alphas #
   calib.dat = aggregate(ind ~ ovrl_ind*alpha, data=resamp_calib, FUN="mean")
   # Linear calibration #
-  fn = function(alpha, ind){
+  fn = function(alpha, ind) {
     (sum( alpha*(1-ind)) / sum(alpha^2))^(-1)
   }
   alpha.ovrl_c = with(calib.dat[calib.dat$ovrl_ind==1,], fn(alpha, ind) )
@@ -171,13 +173,13 @@ calibrate_alpha = function(resamp_calib, alpha_ovrl, alpha_s){
   return(out)
 }
 ### Resampling metric calculation ####
-resamp_metrics = function(param.dat, resamp_param, resamp_calib, resample){
+resamp_metrics = function(param.dat, resamp_param, resamp_calib, resample) {
 
   calibrate=FALSE
   alpha_ovrl = unique(param.dat$alpha[param.dat$Subgrps==0])
   alpha_s = unique(param.dat$alpha[param.dat$Subgrps>0])
   ## Calculate calibrated alpha ##
-  if (!is.null(resamp_calib)){
+  if (!is.null(resamp_calib)) {
     alpha_c = calibrate_alpha(resamp_calib=resamp_calib, alpha_ovrl=alpha_ovrl,
                               alpha_s=alpha_s)
     alpha.ovrl_c = alpha_c$alpha_calib[alpha_c$ovrl_ind==1]
@@ -187,36 +189,37 @@ resamp_metrics = function(param.dat, resamp_param, resamp_calib, resample){
     calibrate=TRUE
   }
   # Loop through estimands #
-  looper = function(e){
+  looper = function(e) {
     final_ests = param.dat[param.dat$estimand==e,]
     final_ests$est_resamp = NA
     final_ests$SE_resamp = NA
-    if (resample=="Permutation"){
+    if (resample=="Permutation") {
       final_ests$pval_perm = NA
     }
-    if (resample=="Bootstrap"){
-      final_ests = final_ests %>% mutate(bias.boot = NA, LCL.pct = NA, UCL.pct= NA)
-      if (calibrate){
+    if (resample=="Bootstrap") {
+      final_ests = final_ests %>% 
+        mutate(bias.boot = NA, LCL.pct = NA, UCL.pct= NA, `Prob(>0)`=NA)
+      if (calibrate) {
         final_ests$LCL.calib = with(final_ests, est - qt(1-alpha_c/2, df=N-1)*SE)
         final_ests$UCL.calib = with(final_ests, est + qt(1-alpha_c/2, df=N-1)*SE)
       }
     }
-    for (sub in unique(final_ests$Subgrps)){
-      if (sub<=0){ alpha = alpha_ovrl }
-      if (sub>0 ){ alpha = alpha_s }
+    for (sub in unique(final_ests$Subgrps)) {
+      if (sub<=0) { alpha = alpha_ovrl }
+      if (sub>0 ) { alpha = alpha_s }
       hold = final_ests[final_ests$Subgrps==sub,]
       hold.R = resamp_param[resamp_param$Subgrps==sub & resamp_param$estimand==e,]
       est0 = hold$est
       est.vec = hold.R$est
       ## Permutation (est, SE, p-value) ##
-      if (resample=="Permutation"){
+      if (resample=="Permutation") {
         final_ests$est_resamp[final_ests$Subgrps==sub] = mean(hold.R$est, na.rm=TRUE)
         final_ests$SE_resamp[final_ests$Subgrps==sub] = sd( est.vec, na.rm=TRUE)
         final_ests$pval_perm[final_ests$Subgrps==sub] =
           (sum(abs(est.vec)>abs(est0), na.rm=TRUE) + 1 ) / (length(na.omit(est.vec))+1)
       }
       ## Bootstrap (smoothed est, SE, bias, pct CI) ##
-      if (resample=="Bootstrap"){
+      if (resample=="Bootstrap") {
         final_ests$est_resamp[final_ests$Subgrps==sub] = mean(hold.R$est, na.rm=TRUE)
         final_ests$SE_resamp[final_ests$Subgrps==sub] = sd( est.vec, na.rm=TRUE)
         final_ests$bias.boot[final_ests$Subgrps==sub] = mean(hold.R$bias, na.rm=TRUE)
@@ -224,9 +227,10 @@ resamp_metrics = function(param.dat, resamp_param, resamp_calib, resample){
           quantile(est.vec, probs=c(alpha/2, (1-alpha/2)), na.rm = TRUE) )
         final_ests$LCL.pct[final_ests$Subgrps==sub] = quants[1]
         final_ests$UCL.pct[final_ests$Subgrps==sub] = quants[2]
+        final_ests$`Prob(>0)`[final_ests$Subgrps==sub] = mean(hold.R$est>0, na.rm=TRUE)
       }
       ## Cross-validation (Cross-fitting estimate) ##
-      if (resample=="CV"){
+      if (resample=="CV") {
         N.tot = sum(hold.R$N)
         est.CF <- with(hold.R, weighted.mean(est, N))
         SE.CF <- with(hold.R, sqrt(  sum( (N/N.tot)^2*SE^2) ) )
@@ -244,16 +248,16 @@ resamp_metrics = function(param.dat, resamp_param, resamp_calib, resample){
 }
 
 #### Weibull Mob Functions ###
-wbreg <- function(y, x, start = NULL, weights = NULL, offset = NULL, ...){
+wbreg <- function(y, x, start = NULL, weights = NULL, offset = NULL, ...) {
   survreg(y ~ 0 + x, weights = weights, dist = "weibull", ...)
 }
-logLik.survreg <- function(object, ...){
+logLik.survreg <- function(object, ...) {
   structure(object$loglik[2], df = sum(object$df), class = "logLik")
 }
 
 ### RMST Estimation: based on survRM2 ###
-rmst_single = function(time, status, tau=NULL){
-  if (is.null(tau)){
+rmst_single = function(time, status, tau=NULL) {
+  if (is.null(tau)) {
     tau = max(time)
   }
   km = survfit(Surv(time, status) ~ 1)
@@ -274,13 +278,13 @@ rmst_single = function(time, status, tau=NULL){
 
 ## P-value converter ##
 pval_convert <- function(p_value) {
-  if (is.na(p_value)){ return( "" )}
+  if (is.na(p_value)) { return( "" )}
   if (p_value < 0.001) return(c("p<0.001"))
   if (p_value >= 0.001) return(paste("p=",(round(p_value,3)),sep=""))
   else return("")
 }
 ## RMST calculator: tau is RMST truncation time ##
-rmst_calc <- function(time, surv, tau){
+rmst_calc <- function(time, surv, tau) {
   idx = time <= tau
   id.time = sort(c(time[idx], tau))
   id.surv = surv[idx]
@@ -289,3 +293,45 @@ rmst_calc <- function(time, surv, tau){
   rmst = sum(areas)
   return(rmst)
 }
+## Probability Calculator (input desired threshold and PRISM.fit) ##
+prob_calculator <- function(fit, thres=">0") {
+  
+  param.dat <- fit$param.dat
+  thres.name <- paste("Prob(",thres, ")", sep="")
+  # Stop if threshold has already been computed #
+  if (thres.name %in% colnames(param.dat)) {
+    return(fit)
+  }
+  dir <- substr(thres, 1, 1)
+  numb <- substr(thres, 2, nchar(thres))
+  if (suppressWarnings(is.na(as.numeric(numb)))) {
+    numb <- substr(thres, 3, nchar(thres))
+  }
+  numb <- as.numeric(numb)
+  # Check resampling / bayesian #
+  resamp <- ifelse(is.null(fit$resamp.dist), FALSE, TRUE)
+  bayes <- ifelse(is.null(fit$bayes.fun), FALSE, TRUE)
+  # Normal Approx (if no normal/bayes)
+  if (!resamp & !bayes) {
+    prob.est <- with(param.dat, 1-pnorm(numb, mean=est, sd=SE))
+    if (dir=="<") prob.est <- 1-prob.est
+    param.dat$prob.est <- prob.est
+  }
+  # Bayes #
+  if (!resamp & bayes) {
+    prob.est <- with(param.dat, 1-pnorm(numb, mean=est.bayes, sd=SE.bayes))
+    if (dir=="<") prob.est <- 1-prob.est
+    param.dat$prob.est <- prob.est
+  }
+  # Resampling #
+  if (resamp) {
+    rdist <- fit$resamp.dist
+    rdist$prob.est = eval(parse(text=paste("ifelse(rdist$est",thres, ", 1, 0)")))
+    prob.dat <- aggregate(prob.est ~ Subgrps*estimand, data=rdist, FUN="mean")
+    param.dat <- left_join(param.dat, prob.dat, by=c("Subgrps", "estimand"))
+  }
+  # Create column name #
+  colnames(param.dat)[which(colnames(param.dat)=="prob.est")] <- thres.name
+  fit$param.dat <- param.dat
+  return(fit)
+}
\ No newline at end of file
